@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2004, 2017 IBM Corporation and others.
+ * Copyright (c) 2004, 2019 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -11,7 +11,6 @@
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *     Red Hat Inc. - stop generating pre 1.2 JNI code
- * Martin Oberhuber (WindRiver) - [515610] Fix incorrect code for memmove()
  *******************************************************************************/
 package org.eclipse.swt.tools.internal;
 
@@ -263,31 +262,14 @@ void generateNativeMacro(JNIClass clazz) {
 	outputln();
 }
 
-boolean generateGetParameter(JNIMethod method, JNIParameter param, boolean critical, int indent) {
+boolean generateGetParameter(JNIParameter param, boolean critical, int indent) {
 	JNIType paramType = param.getType(), paramType64 = param.getType64();
 	if (paramType.isPrimitive() || isSystemClass(paramType)) return false;
 	String iStr = String.valueOf(param.getParameter());
 	for (int j = 0; j < indent; j++) output("\t");
-	//if(!paramType.isArray() && !paramType.isType("java.lang.String") && param.getFlag(FLAG_NO_IN)) {
-	if("memmove".equals(method.getName()) && param.getFlag(FLAG_NO_IN)) {
-		// Bug 515610: Fix incorect fall-through for memmove().
-		// A similar issue might be present in other cases as well. 
-		// Keep diagnostic code (commented-out) below for diagnosing these.
-		output("if (!arg");
-		output(iStr);
-		output(") goto fail;\n");
-		//output(") {fprintf(stderr,\"fail: method=");
-		//output(method.getName());
-		//output(", arg");
-		//output(iStr);
-		//output("\\n\"); goto fail;}\n");
-		for (int j = 0; j < indent; j++) output("\t");
-		output("if ((lparg");
-	} else {
-		output("if (arg");
-		output(iStr);
-		output(") if ((lparg");
-	}
+	output("if (arg");
+	output(iStr);
+	output(") if ((lparg");
 	output(iStr);
 	output(" = ");
 	boolean isCPP = getCPP();
@@ -523,13 +505,13 @@ boolean generateLocalVars(JNIParameter[] params, JNIType returnType, JNIType ret
 	return needsReturn;
 }
 
-boolean generateGetters(JNIMethod method, JNIParameter[] params) {
+boolean generateGetters(JNIParameter[] params) {
 	boolean genFailTag = false;
 	int criticalCount = 0;
 	for (int i = 0; i < params.length; i++) {
 		JNIParameter param = params[i];
 		if (!isCritical(param)) {
-			genFailTag |= generateGetParameter(method, param, false, 1);
+			genFailTag |= generateGetParameter(param, false, 1);
 		} else {
 			criticalCount++;
 		}
@@ -538,7 +520,7 @@ boolean generateGetters(JNIMethod method, JNIParameter[] params) {
 		for (int i = 0; i < params.length; i++) {
 			JNIParameter param = params[i];
 			if (isCritical(param)) {
-				genFailTag |= generateGetParameter(method, param, true, 2);
+				genFailTag |= generateGetParameter(param, true, 2);
 			}
 		}
 	}
@@ -930,7 +912,7 @@ void generateFunctionBody(JNIMethod method, String function, String function64,
 	} else {
 		boolean needsReturn = generateLocalVars(params, returnType, returnType64);
 		generateEnterExitMacro(method, function, function64, true);
-		boolean genFailTag = generateGetters(method, params);
+		boolean genFailTag = generateGetters(params);
 		if (method.getFlag(FLAG_DYNAMIC)) {
 			generateDynamicFunctionCall(method, params, returnType, returnType64, needsReturn);
 		} else {
