@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2011 IBM Corporation and others.
+ * Copyright (c) 2000, 2019 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -32,6 +32,7 @@ import org.eclipse.swt.internal.cocoa.*;
  * <p>
  * Instances are always displayed in one of the maximized,
  * minimized or normal states:
+ * </p>
  * <ul>
  * <li>
  * When an instance is marked as <em>maximized</em>, the
@@ -55,7 +56,6 @@ import org.eclipse.swt.internal.cocoa.*;
  * of these changes are also possible.
  * </li>
  * </ul>
- * </p>
  * Note: The styles supported by this class must be treated
  * as <em>HINT</em>s, since the window manager for the
  * desktop on which the instance is visible has ultimate
@@ -440,7 +440,7 @@ void reskinChildren (int flags) {
 boolean restoreFocus () {
 	if (savedFocus != null && savedFocus.isDisposed ()) savedFocus = null;
 	if (savedFocus == null) return false;
-	return savedFocus.forceFocus ();
+	return savedFocus.setFocus ();
 }
 
 void saveFocus () {
