@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2017 IBM Corporation and others.
+ * Copyright (c) 2000, 2019 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -41,7 +41,7 @@ import org.eclipse.swt.internal.gtk.*;
  */
 public class MenuItem extends Item {
 	Menu parent, menu;
-	long /*int*/ groupHandle, labelHandle, imageHandle;
+	long groupHandle, labelHandle, imageHandle;
 
 	/** Feature in Gtk: as of Gtk version 3.10 GtkImageMenuItem is deprecated,
 	 * meaning that MenuItems in SWT with images can no longer be GtkImageMenuItems
@@ -54,7 +54,7 @@ public class MenuItem extends Item {
 	 * https://developer.gnome.org/gtk3/stable/GtkImageMenuItem.html#GtkImageMenuItem.description
 	 * Bug 470298
 	 */
-	long /*int*/ boxHandle;
+	long boxHandle;
 
 	int accelerator, userId;
 	String toolTipText;
@@ -145,11 +145,11 @@ public MenuItem (Menu parent, int style, int index) {
 	createWidget (index);
 }
 
-void addAccelerator (long /*int*/ accelGroup) {
+void addAccelerator (long accelGroup) {
 	updateAccelerator (accelGroup, true);
 }
 
-void addAccelerators (long /*int*/ accelGroup) {
+void addAccelerators (long accelGroup) {
 	addAccelerator (accelGroup);
 	if (menu != null) menu.addAccelerators (accelGroup);
 }
@@ -264,14 +264,6 @@ void createHandle (int index) {
 			if (handle == 0) error (SWT.ERROR_NO_HANDLES);
 			break;
 		case SWT.RADIO:
-			if (GTK.GTK_VERSION <= OS.VERSION (3, 10, 8)) {
-				/*
-				 * Bug in Ubuntu <= 14.04 with UBUNTU_MENUPROXY. GTK crashes when
-				 * label of radio button is empty, see https://bugs.eclipse.org/419729 .
-				 * The workaround is to initialize the label with a space.
-				 */
-				buffer = new byte [] { ' ', 0 };
-			}
 			/*
 			* Feature in GTK.  In GTK, radio button must always be part of
 			* a radio button group.  In a GTK radio group, one button is always
@@ -287,85 +279,70 @@ void createHandle (int index) {
 			if (groupHandle == 0) error (SWT.ERROR_NO_HANDLES);
 			OS.g_object_ref (groupHandle);
 			OS.g_object_ref_sink (groupHandle);
-			long /*int*/ group = GTK.gtk_radio_menu_item_get_group (groupHandle);
-			if (GTK.GTK3) {
-				handle = GTK.gtk_radio_menu_item_new (group);
-				if (handle == 0) error (SWT.ERROR_NO_HANDLES);
+			long group = GTK.gtk_radio_menu_item_get_group (groupHandle);
+			handle = GTK.gtk_radio_menu_item_new (group);
+			if (handle == 0) error (SWT.ERROR_NO_HANDLES);
 
-				labelHandle = GTK.gtk_accel_label_new (buffer);
-				if (labelHandle == 0) error (SWT.ERROR_NO_HANDLES);
+			labelHandle = GTK.gtk_accel_label_new (buffer);
+			if (labelHandle == 0) error (SWT.ERROR_NO_HANDLES);
 
-				boxHandle = gtk_box_new (GTK.GTK_ORIENTATION_HORIZONTAL, false, 6);
-				if (boxHandle == 0) error (SWT.ERROR_NO_HANDLES);
+			boxHandle = gtk_box_new (GTK.GTK_ORIENTATION_HORIZONTAL, false, 6);
+			if (boxHandle == 0) error (SWT.ERROR_NO_HANDLES);
 
-				if (OS.SWT_PADDED_MENU_ITEMS) {
-					imageHandle = GTK.gtk_image_new ();
-					if (imageHandle == 0) error (SWT.ERROR_NO_HANDLES);
-				}
-			} else { // Gtk2
-				handle = GTK.gtk_radio_menu_item_new_with_label (group, buffer);
+			if (OS.SWT_PADDED_MENU_ITEMS) {
+				imageHandle = GTK.gtk_image_new ();
+				if (imageHandle == 0) error (SWT.ERROR_NO_HANDLES);
 			}
 			break;
 		case SWT.CHECK:
-			if (GTK.GTK3) {
-				handle = GTK.gtk_check_menu_item_new ();
-				if (handle == 0) error (SWT.ERROR_NO_HANDLES);
+			handle = GTK.gtk_check_menu_item_new ();
+			if (handle == 0) error (SWT.ERROR_NO_HANDLES);
 
-				labelHandle = GTK.gtk_accel_label_new (buffer);
-				if (labelHandle == 0) error (SWT.ERROR_NO_HANDLES);
+			labelHandle = GTK.gtk_accel_label_new (buffer);
+			if (labelHandle == 0) error (SWT.ERROR_NO_HANDLES);
 
-				boxHandle = gtk_box_new (GTK.GTK_ORIENTATION_HORIZONTAL, false, 6);
-				if (boxHandle == 0) error (SWT.ERROR_NO_HANDLES);
+			boxHandle = gtk_box_new (GTK.GTK_ORIENTATION_HORIZONTAL, false, 6);
+			if (boxHandle == 0) error (SWT.ERROR_NO_HANDLES);
 
-				if (OS.SWT_PADDED_MENU_ITEMS) {
-					imageHandle = GTK.gtk_image_new ();
-					if (imageHandle == 0) error (SWT.ERROR_NO_HANDLES);
-				}
-			} else { // Gtk2
-				handle = GTK.gtk_check_menu_item_new_with_label (buffer);
-				if (handle == 0) error (SWT.ERROR_NO_HANDLES);
+			if (OS.SWT_PADDED_MENU_ITEMS) {
+				imageHandle = GTK.gtk_image_new ();
+				if (imageHandle == 0) error (SWT.ERROR_NO_HANDLES);
 			}
 			break;
 		// This case now needs to be handled due to double padding. When double padded
 		// menus are used, the "head" menu item (such as File, Edit, Help, etc.) should
 		// not be padded. We only care about this in Gtk3.
 		case SWT.CASCADE:
-			if (GTK.GTK3) {
-				handle = GTK.gtk_menu_item_new ();
-				if (handle == 0) error (SWT.ERROR_NO_HANDLES);
+			handle = GTK.gtk_menu_item_new ();
+			if (handle == 0) error (SWT.ERROR_NO_HANDLES);
 
-				labelHandle = GTK.gtk_accel_label_new (buffer);
-				if (labelHandle == 0) error (SWT.ERROR_NO_HANDLES);
+			labelHandle = GTK.gtk_accel_label_new (buffer);
+			if (labelHandle == 0) error (SWT.ERROR_NO_HANDLES);
 
-				boxHandle = gtk_box_new (GTK.GTK_ORIENTATION_HORIZONTAL, false, 6);
-				if (boxHandle == 0) error (SWT.ERROR_NO_HANDLES);
-				if ((parent.style & bits) == SWT.BAR) {
-					break;
-				}
-				if (OS.SWT_PADDED_MENU_ITEMS) {
-					imageHandle = GTK.gtk_image_new ();
-					if (imageHandle == 0) error (SWT.ERROR_NO_HANDLES);
-				}
+			boxHandle = gtk_box_new (GTK.GTK_ORIENTATION_HORIZONTAL, false, 6);
+			if (boxHandle == 0) error (SWT.ERROR_NO_HANDLES);
+			if ((parent.style & bits) == SWT.BAR) {
 				break;
 			}
+			if (OS.SWT_PADDED_MENU_ITEMS) {
+				imageHandle = GTK.gtk_image_new ();
+				if (imageHandle == 0) error (SWT.ERROR_NO_HANDLES);
+			}
+			break;
 		case SWT.PUSH:
 		default:
-			if (GTK.GTK3) {
-				handle = GTK.gtk_menu_item_new ();
-				if (handle == 0) error (SWT.ERROR_NO_HANDLES);
+			handle = GTK.gtk_menu_item_new ();
+			if (handle == 0) error (SWT.ERROR_NO_HANDLES);
 
-				labelHandle = GTK.gtk_accel_label_new (buffer);
-				if (labelHandle == 0) error (SWT.ERROR_NO_HANDLES);
+			labelHandle = GTK.gtk_accel_label_new (buffer);
+			if (labelHandle == 0) error (SWT.ERROR_NO_HANDLES);
 
-				boxHandle = gtk_box_new (GTK.GTK_ORIENTATION_HORIZONTAL, false, 6);
-				if (boxHandle == 0) error (SWT.ERROR_NO_HANDLES);
+			boxHandle = gtk_box_new (GTK.GTK_ORIENTATION_HORIZONTAL, false, 6);
+			if (boxHandle == 0) error (SWT.ERROR_NO_HANDLES);
 
-				if (OS.SWT_PADDED_MENU_ITEMS) {
-					imageHandle = GTK.gtk_image_new ();
-					if (imageHandle == 0) error (SWT.ERROR_NO_HANDLES);
-				}
-			} else { // Gtk2
-				handle = GTK.gtk_image_menu_item_new_with_label (buffer);
+			if (OS.SWT_PADDED_MENU_ITEMS) {
+				imageHandle = GTK.gtk_image_new ();
+				if (imageHandle == 0) error (SWT.ERROR_NO_HANDLES);
 			}
 			break;
 	}
@@ -383,7 +360,7 @@ void createHandle (int index) {
 		} else {
 			GTK.gtk_misc_set_alignment(labelHandle, 0, 0);
 		}
-		GTK.gtk_box_pack_end (boxHandle, labelHandle, true, true, 0);
+		gtk_box_pack_end (boxHandle, labelHandle, true, true, 0);
 		GTK.gtk_widget_show (labelHandle);
 	}
 	if (boxHandle != 0) {
@@ -396,7 +373,7 @@ void createHandle (int index) {
 		}
 		GTK.gtk_accel_label_set_accel_widget (labelHandle, 0);
 	}
-	long /*int*/ parentHandle = parent.handle;
+	long parentHandle = parent.handle;
 	boolean enabled = GTK.gtk_widget_get_sensitive (parentHandle);
 	if (!enabled) GTK.gtk_widget_set_sensitive (parentHandle, true);
 	GTK.gtk_menu_shell_insert (parentHandle, handle, index);
@@ -417,7 +394,6 @@ void fixMenus (Decorations newParent) {
  *
  * @return the accelerator or 0
  *
- * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -428,7 +404,7 @@ public int getAccelerator () {
 	return accelerator;
 }
 
-long /*int*/ getAccelGroup () {
+long getAccelGroup () {
 	Menu menu = parent;
 	while (menu != null && menu.cascade != null) {
 		menu = menu.cascade.parent;
@@ -568,7 +544,7 @@ public String getToolTipText () {
 }
 
 @Override
-long /*int*/ gtk_activate (long /*int*/ widget) {
+long gtk_activate (long widget) {
 	if ((style & SWT.CASCADE) != 0 && menu != null) return 0;
 	/*
 	* Bug in GTK.  When an ancestor menu is disabled and
@@ -588,14 +564,14 @@ long /*int*/ gtk_activate (long /*int*/ widget) {
 }
 
 @Override
-long /*int*/ gtk_select (long /*int*/ item) {
+long gtk_select (long item) {
 	parent.selectedItem = this;
 	sendEvent (SWT.Arm);
 	return 0;
 }
 
 @Override
-long /*int*/ gtk_show_help (long /*int*/ widget, long /*int*/ helpType) {
+long gtk_show_help (long widget, long helpType) {
 	boolean handled = hooks (SWT.Help);
 	if (handled) {
 		postEvent (SWT.Help);
@@ -658,7 +634,7 @@ void releaseParent () {
 @Override
 void releaseWidget () {
 	super.releaseWidget ();
-	long /*int*/ accelGroup = getAccelGroup ();
+	long accelGroup = getAccelGroup ();
 	if (accelGroup != 0) removeAccelerator (accelGroup);
 	if (groupHandle != 0) OS.g_object_unref (groupHandle);
 	groupHandle = 0;
@@ -666,11 +642,11 @@ void releaseWidget () {
 	parent = null;
 }
 
-void removeAccelerator (long /*int*/ accelGroup) {
+void removeAccelerator (long accelGroup) {
 	updateAccelerator (accelGroup, false);
 }
 
-void removeAccelerators (long /*int*/ accelGroup) {
+void removeAccelerators (long accelGroup) {
 	removeAccelerator (accelGroup);
 	if (menu != null) menu.removeAccelerators (accelGroup);
 }
@@ -774,7 +750,6 @@ void selectRadio () {
  *
  * @param accelerator an integer that is the bit-wise OR of masks and a key
  *
- * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -783,7 +758,7 @@ void selectRadio () {
 public void setAccelerator (int accelerator) {
 	checkWidget();
 	if (this.accelerator == accelerator) return;
-	long /*int*/ accelGroup = getAccelGroup ();
+	long accelGroup = getAccelGroup ();
 	if (accelGroup != 0) removeAccelerator (accelGroup);
 	this.accelerator = accelerator;
 	if (accelGroup != 0) addAccelerator (accelGroup);
@@ -805,7 +780,7 @@ public void setAccelerator (int accelerator) {
 public void setEnabled (boolean enabled) {
 	checkWidget();
 	if (GTK.gtk_widget_get_sensitive (handle) == enabled) return;
-	long /*int*/ accelGroup = getAccelGroup ();
+	long accelGroup = getAccelGroup ();
 	if (accelGroup != 0) removeAccelerator (accelGroup);
 	GTK.gtk_widget_set_sensitive (handle, enabled);
 	if (accelGroup != 0) addAccelerator (accelGroup);
@@ -858,59 +833,59 @@ public void setImage (Image image) {
 		ImageList imageList = parent.imageList;
 		if (imageList == null) imageList = parent.imageList = new ImageList ();
 		int imageIndex = imageList.indexOf (image);
+		long pixbuf = 0;
 		if (imageIndex == -1) {
 			imageIndex = imageList.add (image);
-		} else {
-			imageList.put (imageIndex, image);
-		}
-		long /*int*/ pixbuf = imageList.getPixbuf (imageIndex);
-		if (GTK.GTK3) {
+			pixbuf = imageList.getPixbuf (imageIndex);
+			/*
+			 * Bug in GTK the default renderer does scale again on pixbuf.
+			 * Need to scaledown here and no need to scaledown id device scale is 1
+			 *
+			 * We should resize pixbuf and update pixbuf array in the imagelist only if
+			 * the image is added to the imagelist in this call. If the image is already
+			 * add imageList.getPixbuf returns resized pixbuf.
+			 */
 			if (DPIUtil.useCairoAutoScale()) {
 				Rectangle imgSize = image.getBounds();
-				long /*int*/ scaledPixbuf = GDK.gdk_pixbuf_scale_simple(pixbuf, imgSize.width, imgSize.height, GDK.GDK_INTERP_BILINEAR);
+				long scaledPixbuf = GDK.gdk_pixbuf_scale_simple(pixbuf, imgSize.width, imgSize.height, GDK.GDK_INTERP_BILINEAR);
 				if (scaledPixbuf !=0) {
 					pixbuf = scaledPixbuf;
 				}
-			}
-
-			if (!GTK.GTK_IS_MENU_ITEM (handle)) return;
-			if (OS.SWT_PADDED_MENU_ITEMS && imageHandle != 0) {
-				GTK.gtk_image_set_from_pixbuf(imageHandle, pixbuf);
-			} else {
-				if (imageHandle == 0 && boxHandle != 0) {
-					imageHandle = GTK.gtk_image_new_from_pixbuf (pixbuf);
-					GTK.gtk_container_add (boxHandle, imageHandle);
-					GTK.gtk_box_reorder_child (boxHandle, imageHandle, 0);
-				} else {
-					GTK.gtk_image_set_from_pixbuf(imageHandle, pixbuf);
-				}
-				if (boxHandle == 0) error (SWT.ERROR_NO_HANDLES);
+				imageList.replacePixbuf(imageIndex, pixbuf);
 			}
 		} else {
-			if (!GTK.GTK_IS_IMAGE_MENU_ITEM (handle)) return;
-			imageHandle = GTK.gtk_image_new_from_pixbuf (pixbuf);
-			GTK.gtk_image_menu_item_set_image (handle, imageHandle);
+			imageList.put (imageIndex, image);
+			pixbuf = imageList.getPixbuf (imageIndex);
+		}
+
+		if (!GTK.GTK_IS_MENU_ITEM (handle)) return;
+		if (OS.SWT_PADDED_MENU_ITEMS && imageHandle != 0) {
+			GTK.gtk_image_set_from_pixbuf(imageHandle, pixbuf);
+		} else {
+			if (imageHandle == 0 && boxHandle != 0) {
+				imageHandle = GTK.gtk_image_new_from_pixbuf (pixbuf);
+				GTK.gtk_container_add (boxHandle, imageHandle);
+				GTK.gtk_box_reorder_child (boxHandle, imageHandle, 0);
+			} else {
+				GTK.gtk_image_set_from_pixbuf(imageHandle, pixbuf);
 			}
+			if (boxHandle == 0) error (SWT.ERROR_NO_HANDLES);
+		}
 		if (imageHandle == 0) error (SWT.ERROR_NO_HANDLES);
 		GTK.gtk_widget_show (imageHandle);
 	} else {
-		if (GTK.GTK3) {
-			if (imageHandle != 0 && boxHandle != 0) {
-				if (OS.SWT_PADDED_MENU_ITEMS) {
-					GTK.gtk_container_remove(boxHandle, imageHandle);
-					imageHandle = GTK.gtk_image_new ();
-					if (imageHandle == 0) error (SWT.ERROR_NO_HANDLES);
-					GTK.gtk_image_set_pixel_size (imageHandle, 16);
-					GTK.gtk_container_add (boxHandle, imageHandle);
-					GTK.gtk_widget_show (imageHandle);
-				} else {
-					GTK.gtk_container_remove(boxHandle, imageHandle);
-					imageHandle = 0;
-				}
+		if (imageHandle != 0 && boxHandle != 0) {
+			if (OS.SWT_PADDED_MENU_ITEMS) {
+				GTK.gtk_container_remove(boxHandle, imageHandle);
+				imageHandle = GTK.gtk_image_new ();
+				if (imageHandle == 0) error (SWT.ERROR_NO_HANDLES);
+				GTK.gtk_image_set_pixel_size (imageHandle, 16);
+				GTK.gtk_container_add (boxHandle, imageHandle);
+				GTK.gtk_widget_show (imageHandle);
+			} else {
+				GTK.gtk_container_remove(boxHandle, imageHandle);
+				imageHandle = 0;
 			}
-		} else {
-			if (!GTK.GTK_IS_IMAGE_MENU_ITEM (handle)) return;
-			GTK.gtk_image_menu_item_set_image (handle, 0);
 		}
 	}
 }
@@ -959,7 +934,7 @@ public void setMenu (Menu menu) {
 	/* Assign the new menu */
 	Menu oldMenu = this.menu;
 	if (oldMenu == menu) return;
-	long /*int*/ accelGroup = getAccelGroup ();
+	long accelGroup = getAccelGroup ();
 	if (accelGroup != 0) removeAccelerators (accelGroup);
 	if (oldMenu != null) {
 		oldMenu.cascade = null;
@@ -1063,34 +1038,20 @@ public void setText (String string) {
 	if ((style & SWT.SEPARATOR) != 0) return;
 	if (text.equals (string)) return;
 	super.setText (string);
-	String accelString = "";
 	int index = string.indexOf ('\t');
 	if (index != -1) {
-		boolean isRTL = (parent.style & SWT.RIGHT_TO_LEFT) != 0;
-		accelString = (isRTL? "" : "  ") + string.substring (index+1, string.length()) + (isRTL? "  " : "");
 		string = string.substring (0, index);
 	}
 	char [] chars = fixMnemonic (string);
 	byte [] buffer = Converter.wcsToMbcs (chars, true);
-	if (boxHandle == 0 && !GTK.GTK3) {
-		labelHandle = GTK.gtk_bin_get_child (handle);
-	}
 	if (labelHandle != 0 && GTK.GTK_IS_LABEL (labelHandle)) {
 		GTK.gtk_label_set_text_with_mnemonic (labelHandle, buffer);
 		if (GTK.GTK_IS_ACCEL_LABEL (labelHandle)) {
-			if (GTK.GTK3) {
-				if (GTK.GTK_VERSION >= OS.VERSION(3, 6, 0)) {
-					MaskKeysym maskKeysym = getMaskKeysym();
-					if (maskKeysym != null) {
-						GTK.gtk_accel_label_set_accel_widget (labelHandle, handle);
-						GTK.gtk_accel_label_set_accel (labelHandle,
-								maskKeysym.keysym, maskKeysym.mask);
-					}
-				} else {
-					setAccelLabel (labelHandle, accelString);
-				}
-			} else {
-				setAccelLabel (labelHandle, accelString);
+			MaskKeysym maskKeysym = getMaskKeysym();
+			if (maskKeysym != null) {
+				GTK.gtk_accel_label_set_accel_widget (labelHandle, handle);
+				GTK.gtk_accel_label_set_accel (labelHandle,
+						maskKeysym.keysym, maskKeysym.mask);
 			}
 			// A workaround for Ubuntu Unity global menu
 			OS.g_signal_emit_by_name(handle, OS.accel_closures_changed);
@@ -1098,15 +1059,6 @@ public void setText (String string) {
 	}
 }
 
-private void setAccelLabel(long /*int*/ label, String accelString) {
-	byte[] buffer = Converter.wcsToMbcs (accelString, true);
-	long /*int*/ ptr = OS.g_malloc (buffer.length);
-	C.memmove (ptr, buffer, buffer.length);
-	long /*int*/ oldPtr = GTK.GTK_ACCEL_LABEL_GET_ACCEL_STRING (label);
-	GTK.GTK_ACCEL_LABEL_SET_ACCEL_STRING (label, ptr);
-	if (oldPtr != 0) OS.g_free (oldPtr);
-}
-
 /**
  * Sets the receiver's tool tip text to the argument, which
  * may be null indicating that the default tool tip for the
@@ -1125,7 +1077,7 @@ private void setAccelLabel(long /*int*/ label, String accelString) {
  * </p>
  * <p>
  * NOTE: This operation is a hint and behavior is platform specific, on Windows
- * for CJK-style mnemonics of the form " (&C)" at the end of the tooltip text
+ * for CJK-style mnemonics of the form " (&amp;C)" at the end of the tooltip text
  * are not shown in tooltip.
  * </p>
  * @param toolTip the new tool tip text (or null)
@@ -1144,7 +1096,7 @@ public void setToolTipText (String toolTip) {
 	this.parent.getShell().setToolTipText (handle, (toolTipText = toolTip));
 }
 
-void updateAccelerator (long /*int*/ accelGroup, boolean add) {
+void updateAccelerator (long accelGroup, boolean add) {
 	if (accelerator == 0 || !getEnabled ()) return;
 	if ((accelerator & SWT.COMMAND) != 0) return;
 	int mask = 0;
@@ -1237,7 +1189,7 @@ boolean updateAcceleratorText (boolean show) {
 	}
 	if (maskKeysym == null) return true;
 	if (maskKeysym.keysym != 0) {
-		long /*int*/ accelGroup = getAccelGroup ();
+		long accelGroup = getAccelGroup ();
 		if (show) {
 			GTK.gtk_widget_add_accelerator (handle, OS.activate, accelGroup, maskKeysym.keysym, maskKeysym.mask, GTK.GTK_ACCEL_VISIBLE);
 		} else {
