@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2017 IBM Corporation and others.
+ * Copyright (c) 2000, 2019 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -33,6 +33,7 @@ import org.eclipse.swt.internal.win32.*;
  * <p>
  * Instances are always displayed in one of the maximized,
  * minimized or normal states:
+ * </p>
  * <ul>
  * <li>
  * When an instance is marked as <em>maximized</em>, the
@@ -56,7 +57,6 @@ import org.eclipse.swt.internal.win32.*;
  * of these changes are also possible.
  * </li>
  * </ul>
- * </p>
  * Note: The styles supported by this class must be treated
  * as <em>HINT</em>s, since the window manager for the
  * desktop on which the instance is visible has ultimate
@@ -683,10 +683,9 @@ public String getText () {
 	checkWidget ();
 	int length = OS.GetWindowTextLength (handle);
 	if (length == 0) return "";
-	/* Use the character encoding for the default locale */
-	TCHAR buffer = new TCHAR (0, length + 1);
+	char [] buffer = new char [length + 1];
 	OS.GetWindowText (handle, buffer, length + 1);
-	return buffer.toString (0, length);
+	return new String (buffer, 0, length);
 }
 
 @Override
@@ -778,7 +777,7 @@ void reskinChildren (int flags) {
 boolean restoreFocus () {
 	if (display.ignoreRestoreFocus) return true;
 	if (savedFocus != null && savedFocus.isDisposed ()) savedFocus = null;
-	if (savedFocus != null && savedFocus.setSavedFocus ()) return true;
+	if (savedFocus != null && savedFocus.setFocus ()) return true;
 	return false;
 }
 
@@ -1519,9 +1518,9 @@ LRESULT WM_ACTIVATE (long /*int*/ wParam, long /*int*/ lParam) {
 	* WM_ACTIVATE messages that come from AWT Windows.
 	*/
 	if (OS.GetParent (lParam) == handle) {
-		TCHAR buffer = new TCHAR (0, 128);
-		OS.GetClassName (lParam, buffer, buffer.length ());
-		String className = buffer.toString (0, buffer.strlen ());
+		char [] buffer = new char [128];
+		int length = OS.GetClassName (lParam, buffer, buffer.length);
+		String className = new String (buffer, 0, length);
 		if (className.equals (Display.AWT_WINDOW_CLASS)) {
 			return LRESULT.ZERO;
 		}
