@@ -63,15 +63,9 @@ public class Group extends Composite {
 		GroupProc = lpWndClass.lpfnWndProc;
 		long /*int*/ hInstance = OS.GetModuleHandle (null);
 		if (!OS.GetClassInfo (hInstance, GroupClass, lpWndClass)) {
-			long /*int*/ hHeap = OS.GetProcessHeap ();
 			lpWndClass.hInstance = hInstance;
 			lpWndClass.style &= ~(OS.CS_HREDRAW | OS.CS_VREDRAW);
-			int byteCount = GroupClass.length () * TCHAR.sizeof;
-			long /*int*/ lpszClassName = OS.HeapAlloc (hHeap, OS.HEAP_ZERO_MEMORY, byteCount);
-			OS.MoveMemory (lpszClassName, GroupClass, byteCount);
-			lpWndClass.lpszClassName = lpszClassName;
-			OS.RegisterClass (lpWndClass);
-			OS.HeapFree (hHeap, 0, lpszClassName);
+			OS.RegisterClass (GroupClass, lpWndClass);
 		}
 	}
 
@@ -155,14 +149,14 @@ protected void checkSubclass () {
 		* If the group has text, and the text is wider than the
 		* client area, pad the width so the text is not clipped.
 		*/
-		TCHAR buffer = new TCHAR (getCodePage (), string == null ? text : string, true);
+		char [] buffer = (string == null ? text : string).toCharArray ();
 		long /*int*/ newFont, oldFont = 0;
 		long /*int*/ hDC = OS.GetDC (handle);
 		newFont = OS.SendMessage (handle, OS.WM_GETFONT, 0, 0);
 		if (newFont != 0) oldFont = OS.SelectObject (hDC, newFont);
 		RECT rect = new RECT ();
 		int flags = OS.DT_CALCRECT | OS.DT_SINGLELINE;
-		OS.DrawText (hDC, buffer, -1, rect, flags);
+		OS.DrawText (hDC, buffer, buffer.length, rect, flags);
 		if (newFont != 0) OS.SelectObject (hDC, oldFont);
 		OS.ReleaseDC (handle, hDC);
 		int offsetY = OS.IsAppThemed () ? 0 : 1;
@@ -178,7 +172,7 @@ protected void checkSubclass () {
 	long /*int*/ hDC = OS.GetDC (handle);
 	newFont = OS.SendMessage (handle, OS.WM_GETFONT, 0, 0);
 	if (newFont != 0) oldFont = OS.SelectObject (hDC, newFont);
-	TEXTMETRIC tm = OS.IsUnicode ? (TEXTMETRIC) new TEXTMETRICW () : new TEXTMETRICA ();
+	TEXTMETRIC tm = new TEXTMETRIC ();
 	OS.GetTextMetrics (hDC, tm);
 	if (newFont != 0) OS.SelectObject (hDC, oldFont);
 	OS.ReleaseDC (handle, hDC);
@@ -254,7 +248,7 @@ String fixText (boolean enabled) {
 	long /*int*/ hDC = OS.GetDC (handle);
 	newFont = OS.SendMessage (handle, OS.WM_GETFONT, 0, 0);
 	if (newFont != 0) oldFont = OS.SelectObject (hDC, newFont);
-	TEXTMETRIC tm = OS.IsUnicode ? (TEXTMETRIC) new TEXTMETRICW () : new TEXTMETRICA ();
+	TEXTMETRIC tm = new TEXTMETRIC ();
 	OS.GetTextMetrics (hDC, tm);
 	if (newFont != 0) OS.SelectObject (hDC, oldFont);
 	OS.ReleaseDC (handle, hDC);
@@ -387,7 +381,7 @@ public void setFont (Font font) {
  * Sets the receiver's text, which is the string that will
  * be displayed as the receiver's <em>title</em>, to the argument,
  * which may not be null. The string may include the mnemonic character.
- * </p>
+ * <p>
  * Mnemonics are indicated by an '&amp;' that causes the next
  * character to be the mnemonic.  When the user presses a
  * key sequence that matches the mnemonic, focus is assigned
@@ -508,7 +502,7 @@ LRESULT WM_PAINT (long /*int*/ wParam, long /*int*/ lParam) {
 
 	if (hasCustomForeground() && text.length () != 0) {
 		String string = fixText (false);
-		TCHAR buffer = new TCHAR (getCodePage (), string == null ? text : string, false);
+		char [] buffer = (string == null ? text : string).toCharArray ();
 
 		// We cannot use BeginPaint and EndPaint, because that removes the group border
 		long /*int*/ hDC = OS.GetDC(handle);
@@ -520,13 +514,13 @@ LRESULT WM_PAINT (long /*int*/ wParam, long /*int*/ lParam) {
 		newFont = OS.SendMessage (handle, OS.WM_GETFONT, 0, 0);
 		if (newFont != 0) oldFont = OS.SelectObject (hDC, newFont);
 
-		OS.DrawText(hDC, buffer, buffer.length(), rect, OS.DT_SINGLELINE | OS.DT_LEFT | OS.DT_TOP | OS.DT_CALCRECT);
+		OS.DrawText(hDC, buffer, buffer.length, rect, OS.DT_SINGLELINE | OS.DT_LEFT | OS.DT_TOP | OS.DT_CALCRECT);
 		// The calculated rectangle is a little bit too small. Italic fonts would show some small part in the default color.
 		rect.right += CLIENT_INSET;
 		drawBackground(hDC, rect);
 		OS.SetBkMode(hDC, OS.TRANSPARENT);
 		OS.SetTextColor(hDC, getForegroundPixel());
-		OS.DrawText(hDC, buffer, buffer.length(), rect, OS.DT_SINGLELINE | OS.DT_LEFT | OS.DT_TOP);
+		OS.DrawText(hDC, buffer, buffer.length, rect, OS.DT_SINGLELINE | OS.DT_LEFT | OS.DT_TOP);
 
 		if (newFont != 0) OS.SelectObject (hDC, oldFont);
 		OS.ReleaseDC(handle, hDC);
