@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2019 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -19,22 +19,23 @@ package org.eclipse.swt.snippets;
  * For a list of all SWT example snippets see
  * http://www.eclipse.org/swt/snippets/
  */
-import org.eclipse.swt.graphics.*;
+import org.eclipse.swt.*;
 import org.eclipse.swt.widgets.*;
 
 public class Snippet13 {
 
-public static void main (String [] args) {
-	Display display = new Display ();
-	Shell shell = new Shell (display);
-	shell.open ();
-	GC gc = new GC (shell);
-	gc.setLineWidth (4);
-	gc.drawRectangle (20, 20, 100, 100);
-	gc.dispose ();
-	while (!shell.isDisposed ()) {
-		if (!display.readAndDispatch ()) display.sleep ();
+public static void main(String[] args) {
+	Display display = new Display();
+	Shell shell = new Shell(display);
+	shell.addListener(SWT.Paint, e -> {
+		e.gc.setLineWidth(4);
+		e.gc.drawRectangle(20, 20, 100, 100);
+	});
+	shell.open();
+	while (!shell.isDisposed()) {
+		if (!display.readAndDispatch())
+			display.sleep();
 	}
-	display.dispose ();
+	display.dispose();
 }
 }
