@@ -26,10 +26,12 @@ import org.eclipse.swt.widgets.*;
  * http://www.eclipse.org/swt/snippets/
  */
 public class Snippet366 {
-	  static Display display = new Display();
-	  static Shell shell = new Shell(display);
+	  static Display display;
+	  static Shell shell;
 
 	public static void main (String [] args) {
+		display = new Display();
+		shell = new Shell(display);
 		shell.setLayout (new RowLayout ());
 
 		makeArrowGroup ();
