@@ -520,8 +520,20 @@ public void test_isEnabled() {
 	assertFalse(control.isEnabled());
 }
 @Test
-public void test_isFocusControl() {
+public void test_isFocusControl() throws InterruptedException {
 	assertFalse(control.isFocusControl());
+	if (SwtTestUtil.isCocoa) {
+		//TODO Fix Cocoa failure.
+		if (SwtTestUtil.verbose) {
+			System.out.println("Excluded test_isFocusControl(org.eclipse.swt.tests.junit.Test_org_eclipse_swt_widgets_Control)");
+		}
+		return;
+	}
+	shell.open();
+	// Wait for the shell to become active
+	processEvents(500, () -> shell.getDisplay().getActiveShell() == shell);
+	assertEquals(shell, shell.getDisplay().getActiveShell());
+	assertEquals("Unexpected focus", control.forceFocus(), control.isFocusControl());
 }
 @Test
 public void test_isReparentable() {
