@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2017 IBM Corporation and others.
+ * Copyright (c) 2000, 2019 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -16,6 +16,7 @@ package org.eclipse.swt.tests.junit;
 import static org.junit.Assert.assertArrayEquals;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNotEquals;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertTrue;
@@ -75,6 +76,7 @@ public void test_ConstructorI() {
 	for (int i = 0; i < cases.length; i++) {
 		newShell = new Shell(cases[i]);
 		assertTrue("a " +i, newShell.getDisplay() == shell.getDisplay());
+		assertNotEquals(0, newShell.getStyle() & cases[i]);
 		newShell.dispose();
 	}
 }
@@ -692,36 +694,39 @@ public void test_activateEventSend() {
 
 /**
  * Regression test for Bug 445900: [GTK] Shell#computeTrim(..) wrong for
- * first invisible shell (editor hovers jump when enriched)
+ * first invisible shell (editor hovers jump when enriched).
+ *
+ * Disabled on Wayland as there is no absolute positioning.
  */
 @Test
 public void test_setBounds() throws Exception {
+	if (SwtTestUtil.isX11) {
+		Rectangle bounds = new Rectangle(100, 200, 200, 200);
+		Rectangle bounds2 = new Rectangle(150, 250, 250, 250);
 
-	Rectangle bounds = new Rectangle(100, 200, 200, 200);
-	Rectangle bounds2 = new Rectangle(150, 250, 250, 250);
+		StringBuilder log = new StringBuilder();
+		int[] styles = { SWT.NO_TRIM, SWT.BORDER, SWT.RESIZE, SWT.TITLE | SWT.BORDER, SWT.TITLE | SWT.RESIZE, SWT.TITLE };
+		for (int i = 0; i < styles.length; i++) {
+			Shell testShell = new Shell(shell, styles[i]);
+			try {
+				testShell.setBounds(bounds);
+				logUnlessEquals(log, i + ".1: style 0x" + Integer.toHexString(styles[i]), bounds, testShell.getBounds());
 
-	StringBuilder log = new StringBuilder();
-	int[] styles = { SWT.NO_TRIM, SWT.BORDER, SWT.RESIZE, SWT.TITLE | SWT.BORDER, SWT.TITLE | SWT.RESIZE, SWT.TITLE };
-	for (int i = 0; i < styles.length; i++) {
-		Shell testShell = new Shell(shell, styles[i]);
-		try {
-			testShell.setBounds(bounds);
-			logUnlessEquals(log, i + ".1: style 0x" + Integer.toHexString(styles[i]), bounds, testShell.getBounds());
+				testShell.open();
+				logUnlessEquals(IS_GTK_BUG_445900 ? System.out : log, i + ".2: style 0x" + Integer.toHexString(styles[i]), bounds, testShell.getBounds());
 
-			testShell.open();
-			logUnlessEquals(IS_GTK_BUG_445900 ? System.out : log, i + ".2: style 0x" + Integer.toHexString(styles[i]), bounds, testShell.getBounds());
+				testShell.setBounds(bounds);
+				logUnlessEquals(log, i + ".3: style 0x" + Integer.toHexString(styles[i]), bounds, testShell.getBounds());
 
-			testShell.setBounds(bounds);
-			logUnlessEquals(log, i + ".3: style 0x" + Integer.toHexString(styles[i]), bounds, testShell.getBounds());
-
-			testShell.setBounds(bounds2);
-			logUnlessEquals(log, i + ".4: style 0x" + Integer.toHexString(styles[i]), bounds2, testShell.getBounds());
-		} finally {
-			testShell.dispose();
+				testShell.setBounds(bounds2);
+				logUnlessEquals(log, i + ".4: style 0x" + Integer.toHexString(styles[i]), bounds2, testShell.getBounds());
+			} finally {
+				testShell.dispose();
+			}
+		}
+		if (log.length() > 0) {
+			Assert.fail(log.toString());
 		}
-	}
-	if (log.length() > 0) {
-		Assert.fail(log.toString());
 	}
 }
 
