@@ -24,6 +24,7 @@ import org.eclipse.swt.events.SegmentListener;
 import org.eclipse.swt.events.SelectionEvent;
 import org.eclipse.swt.events.SelectionListener;
 import org.eclipse.swt.events.VerifyListener;
+import org.eclipse.swt.graphics.Color;
 import org.eclipse.swt.graphics.Font;
 import org.eclipse.swt.graphics.FontData;
 import org.eclipse.swt.graphics.Point;
@@ -613,7 +614,7 @@ public void test_getSelectionText() {
 
 @Test
 public void test_getTabs() {
-	if (SWT.getPlatform().equals("win32")) {
+	if (SWT.getPlatform().equals("win32") || SWT.getPlatform().equals("gtk")) {
 		// API not supported on all platforms
 		text.setTabs(1);
 		assertTrue(":a:", text.getTabs() == 1);
@@ -1154,6 +1155,25 @@ public void test_setFontLorg_eclipse_swt_graphics_Font() {
 	font.dispose();
 }
 
+@Test
+public void test_setForegroundAfterBackground() {
+	makeCleanEnvironment(false);
+	Color gray = text.getDisplay().getSystemColor(SWT.COLOR_DARK_GRAY);
+	Color white = text.getDisplay().getSystemColor(SWT.COLOR_WHITE);
+	int systemColor = SwtTestUtil.isCocoa ? SWT.COLOR_LIST_FOREGROUND : SWT.COLOR_WIDGET_FOREGROUND;
+	Color defaultForeground = text.getDisplay().getSystemColor(systemColor);
+
+	text.setBackground(gray);
+	assertEquals(text.getForeground(), defaultForeground);
+	text.setForeground(white);
+	assertEquals(text.getForeground(), white);
+	assertEquals(text.getBackground(), gray);
+
+	gray.dispose();
+	white.dispose();
+	defaultForeground.dispose();
+}
+
 @Test
 public void test_setOrientationI() {
 	text.setOrientation(SWT.RIGHT_TO_LEFT);
