@@ -28,6 +28,7 @@ import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.TreeListener;
 import org.eclipse.swt.graphics.Color;
 import org.eclipse.swt.layout.FillLayout;
+import org.eclipse.swt.widgets.Display;
 import org.eclipse.swt.widgets.Event;
 import org.eclipse.swt.widgets.Tree;
 import org.eclipse.swt.widgets.TreeColumn;
@@ -411,7 +412,7 @@ public void test_setItemCountI() {
 	new TreeItem (tree, SWT.NULL, 0);
 	assertEquals(1, tree.indexOf(tree.getItems()[1]));
 	assertEquals(8, tree.getItemCount());
-	tree.removeAll();
+	tree.setItemCount(0);
 	assertEquals(0, tree.getItemCount());
 	tree.setItemCount(0);
 	assertEquals(0, tree.getItemCount());
@@ -433,6 +434,14 @@ public void test_setItemCountI() {
 	tree.setItemCount(40);
 	assertEquals(40, tree.getItemCount());
 	tree.getItem(39);
+	tree.setItemCount(0);
+	assertEquals(0, tree.getItemCount());
+	try {
+		tree.getItem(39);
+		fail("No exception thrown for illegal index argument");
+	}
+	catch (IllegalArgumentException e) {
+	}
 }
 
 @Test
@@ -834,6 +843,82 @@ public void test_consistency_DragDetect () {
     consistencyEvent(30, 20, 50, 30, ConsistencyUtility.MOUSE_DRAG, events);
 }
 
+@Test
+public void test_disposeItemNotTriggerSelection() {
+	if (SwtTestUtil.isWindows) {
+		// TODO Fix Windows failure.
+		if (SwtTestUtil.verbose) {
+			System.out
+					.println("Excluded test_disposeItemNotTriggerSelection(org.eclipse.swt.tests.junit.Test_org_eclipse_swt_widgets_Tree).");
+		}
+		return;
+	}
+	Display display = shell.getDisplay();
+	shell.setLayout(new FillLayout());
+	Tree tree = new Tree (shell, SWT.BORDER);
+	for (int i=0; i<4; i++) {
+		TreeItem iItem = new TreeItem (tree, 0);
+		iItem.setText ("TreeItem (0) -" + i);
+		for (int j=0; j<4; j++) {
+			TreeItem jItem = new TreeItem (iItem, 0);
+			jItem.setText ("TreeItem (1) -" + j);
+			for (int k=0; k<4; k++) {
+				TreeItem kItem = new TreeItem (jItem, 0);
+				kItem.setText ("TreeItem (2) -" + k);
+				for (int l=0; l<4; l++) {
+					TreeItem lItem = new TreeItem(kItem, 0);
+					lItem.setText ("TreeItem (3) -" + l);
+				}
+			}
+		}
+	}
+	final boolean [] selectionCalled = { false };
+	tree.addListener(SWT.Selection, event -> {
+		selectionCalled [0] = true;
+	});
+
+	final TreeItem firstNode = tree.getItem(0);
+	firstNode.setExpanded(true);
+	tree.setSelection(firstNode.getItem(3));
+
+	shell.setSize(200, 200);
+	shell.open();
+
+	display.timerExec(1000, () -> {
+		if (shell.isDisposed()) {
+			return;
+		}
+
+		final TreeItem[] selection = tree.getSelection();
+		if (selection.length != 1) {
+			return;
+		}
+
+		final TreeItem item = selection[0];
+		final TreeItem parentItem = item.getParentItem();
+		if (parentItem == null) {
+			return;
+		}
+
+		tree.deselectAll();
+		item.dispose();
+
+	});
+
+	long end = System.currentTimeMillis() + 3000;
+	while (!shell.isDisposed() && System.currentTimeMillis() < end) {
+		if (!shell.getDisplay().readAndDispatch ()) {
+			try {
+				Thread.sleep(10);
+			} catch (InterruptedException e) {
+				e.printStackTrace();
+			}
+		}
+	}
+
+	assertFalse(selectionCalled[0]);
+}
+
 @Test
 public void test_Virtual() {
 	tree.dispose();
